"""
CLI interface for the FastAPI Postman Exporter.
"""

import importlib.util
import sys
from pathlib import Path
from typing import Optional

import typer
from fastapi import FastAPI

from .exporter import generate_postman_from_app

app = typer.Typer(
    name="fastapi-postman-exporter",
    help="Export FastAPI applications to Postman collections",
    add_completion=False,
)


def load_fastapi_app(app_path: str, app_name: str) -> FastAPI:
    """
    Dynamically load a FastAPI app from a Python file.
    
    Args:
        app_path: Path to the Python file containing the FastAPI app
        app_name: Name of the FastAPI app variable in the file
        
    Returns:
        The loaded FastAPI app instance
        
    Raises:
        ImportError: If the file cannot be imported or app not found
    """
    app_file = Path(app_path)
    
    if not app_file.exists():
        raise ImportError(f"App file not found: {app_path}")
    
    if not app_file.suffix == ".py":
        raise ImportError(f"App file must be a Python file (.py): {app_path}")
    
    # Load the module
    spec = importlib.util.spec_from_file_location("fastapi_app", app_file)
    if spec is None or spec.loader is None:
        raise ImportError(f"Could not load module from: {app_path}")
    
    module = importlib.util.module_from_spec(spec)
    sys.modules["fastapi_app"] = module
    spec.loader.exec_module(module)
    
    # Get the FastAPI app instance
    if not hasattr(module, app_name):
        raise ImportError(f"App '{app_name}' not found in {app_path}")
    
    app_instance = getattr(module, app_name)
    
    if not isinstance(app_instance, FastAPI):
        raise ImportError(f"'{app_name}' is not a FastAPI app instance")
    
    return app_instance


@app.command()
def export(
    app_path: str = typer.Option(
        ...,
        "--app-path",
        "-p",
        help="Path to the FastAPI app file (e.g., main.py)"
    ),
    app_name: str = typer.Option(
        "app",
        "--app-name",
        "-n",
        help="Name of the FastAPI app object in the file"
    ),
    output: str = typer.Option(
        ...,
        "--output",
        "-o",
        help="Path to export the Postman collection JSON"
    ),
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-v",
        help="Enable verbose output"
    )
) -> None:
    """
    Export a FastAPI application to a Postman collection.
    
    Example:
        fastapi-postman-exporter export --app-path main.py --app-name app --output collection.json
    """
    try:
        if verbose:
            typer.echo(f"Loading FastAPI app from: {app_path}")
            typer.echo(f"App name: {app_name}")
            typer.echo(f"Output path: {output}")
        
        # Load the FastAPI app
        fastapi_app = load_fastapi_app(app_path, app_name)
        
        if verbose:
            typer.echo("FastAPI app loaded successfully")
            typer.echo(f"App title: {fastapi_app.title}")
            typer.echo(f"App version: {fastapi_app.version}")
        
        # Generate Postman collection
        generate_postman_from_app(fastapi_app, output)
        
        typer.echo(f"✅ Successfully exported Postman collection to: {output}")
        
    except ImportError as e:
        typer.echo(f"❌ Error loading FastAPI app: {e}", err=True)
        raise typer.Exit(1)
    except Exception as e:
        typer.echo(f"❌ Error generating Postman collection: {e}", err=True)
        raise typer.Exit(1)


@app.command()
def check() -> None:
    """
    Check if the openapi2postmanv2 CLI tool is installed.
    """
    try:
        from .exporter import check_openapi2postmanv2_installed
        check_openapi2postmanv2_installed()
        typer.echo("✅ openapi2postmanv2 CLI tool is installed and available")
    except RuntimeError as e:
        typer.echo(f"❌ {e}", err=True)
        raise typer.Exit(1)


if __name__ == "__main__":
    app() 