"""
Core exporter functionality for converting FastAPI apps to Postman collections.
"""

import json
import shutil
import subprocess
import tempfile
from pathlib import Path
from typing import Optional

from fastapi import FastAPI


def check_openapi2postmanv2_installed() -> None:
    """
    Check if the openapi2postmanv2 CLI tool is installed.
    
    Raises:
        RuntimeError: If the CLI tool is not found in PATH.
    """
    if shutil.which("openapi2postmanv2") is None:
        raise RuntimeError(
            "openapi2postmanv2 CLI tool is not installed. "
            "Please install it using: npm install -g openapi2postmanv2"
        )


def generate_postman_from_app(app: FastAPI, output_path: str) -> None:
    """
    Generate a Postman collection from a FastAPI application.
    
    Args:
        app: The FastAPI application instance
        output_path: Path where the Postman collection JSON will be saved
        
    Raises:
        RuntimeError: If openapi2postmanv2 CLI is not installed
        subprocess.CalledProcessError: If the conversion process fails
        OSError: If there are file I/O errors
    """
    # Check if the CLI tool is available
    check_openapi2postmanv2_installed()
    
    # Generate OpenAPI spec from FastAPI app
    openapi_spec = app.openapi()
    
    # Create a temporary file for the OpenAPI spec
    with tempfile.NamedTemporaryFile(mode='w', suffix='.json', delete=False) as temp_file:
        temp_file_path = temp_file.name
        json.dump(openapi_spec, temp_file, indent=2)
    
    try:
        # Convert OpenAPI spec to Postman collection using openapi2postmanv2
        result = subprocess.run(
            [
                "openapi2postmanv2",
                "-s", temp_file_path,
                "-o", output_path
            ],
            capture_output=True,
            text=True,
            check=True
        )
        
        print(f"Successfully generated Postman collection at: {output_path}")
        
    except subprocess.CalledProcessError as e:
        error_msg = f"Failed to convert OpenAPI spec to Postman collection: {e.stderr}"
        raise RuntimeError(error_msg) from e
    finally:
        # Clean up temporary file
        try:
            Path(temp_file_path).unlink()
        except OSError:
            pass  # Ignore cleanup errors 